//
//  facebookwdae2eApp.swift
//  facebookwdae2e
//
//  Created by youngfreefjs on 2024/3/20.
//

import SwiftUI

@main
struct facebookwdae2eApp: App {
    var body: some Scene {
        WindowGroup {
            ContentView()
        }
    }
}
