from enum import Enum


UNDER_TEST_BUNDLE_ID = "com.test.cert.TestCert"

UNKNOW_BUNDLE_ID = "com.apple.unknown"

HOME_SCREEN_BUNDLE_ID = "com.apple.springboard"

class ALERT_ENUMS(Enum):
    CONFIRM = "ACCEPT_OR_REJECT_ALERT"
    INPUT = "INPUT_ALERT"
    

